@@ -178,6 +178,9 @@ type (
 		// the key value store in a streaming fashion (on a channel).
 		ListKeys(ctx context.Context, opts ...WatchOpt) (KeyLister, error)
 
+		// ListKeysFiltered ListKeysWithFilters returns a KeyLister for filtered keys in the bucket.
+		ListKeysFiltered(ctx context.Context, filters ...string) (KeyLister, error)
+
 		// History will return all historical values for the key (up to
 		// KeyValueMaxHistory).
 		History(ctx context.Context, key string, opts ...WatchOpt) ([]KeyValueEntry, error)
@@ -787,13 +790,13 @@ func (kl *kvLister) Error() error {
 
 func (js *jetStream) legacyJetStream() (nats.JetStreamContext, error) {
 	opts := make([]nats.JSOpt, 0)
-	if js.apiPrefix != "" {
-		opts = append(opts, nats.APIPrefix(js.apiPrefix))
+	if js.opts.apiPrefix != "" {
+		opts = append(opts, nats.APIPrefix(js.opts.apiPrefix))
 	}
-	if js.clientTrace != nil {
+	if js.opts.clientTrace != nil {
 		opts = append(opts, nats.ClientTrace{
-			RequestSent:      js.clientTrace.RequestSent,
-			ResponseReceived: js.clientTrace.ResponseReceived,
+			RequestSent:      js.opts.clientTrace.RequestSent,
+			ResponseReceived: js.opts.clientTrace.ResponseReceived,
 		})
 	}
 	return js.conn.JetStream(opts...)
@@ -925,7 +928,7 @@ func (kv *kvs) Put(ctx context.Context, key string, value []byte) (uint64, error
 
 	var b strings.Builder
 	if kv.useJSPfx {
-		b.WriteString(kv.js.apiPrefix)
+		b.WriteString(kv.js.opts.apiPrefix)
 	}
 	if kv.putPre != "" {
 		b.WriteString(kv.putPre)
@@ -975,7 +978,7 @@ func (kv *kvs) Update(ctx context.Context, key string, value []byte, revision ui
 
 	var b strings.Builder
 	if kv.useJSPfx {
-		b.WriteString(kv.js.apiPrefix)
+		b.WriteString(kv.js.opts.apiPrefix)
 	}
 	b.WriteString(kv.pre)
 	b.WriteString(key)
@@ -998,7 +1001,7 @@ func (kv *kvs) Delete(ctx context.Context, key string, opts ...KVDeleteOpt) erro
 
 	var b strings.Builder
 	if kv.useJSPfx {
-		b.WriteString(kv.js.apiPrefix)
+		b.WriteString(kv.js.opts.apiPrefix)
 	}
 	if kv.putPre != "" {
 		b.WriteString(kv.putPre)
@@ -1271,6 +1274,36 @@ func (kv *kvs) ListKeys(ctx context.Context, opts ...WatchOpt) (KeyLister, error
 	return kl, nil
 }
 
+// ListKeysWithFilters returns a channel of keys matching the provided filters using WatchFiltered.
+func (kv *kvs) ListKeysFiltered(ctx context.Context, filters ...string) (KeyLister, error) {
+	watcher, err := kv.WatchFiltered(ctx, filters, IgnoreDeletes(), MetaOnly())
+	if err != nil {
+		return nil, err
+	}
+
+	// Reuse the existing keyLister implementation
+	kl := &keyLister{watcher: watcher, keys: make(chan string, 256)}
+
+	go func() {
+		defer close(kl.keys)
+		defer watcher.Stop()
+
+		for {
+			select {
+			case entry := <-watcher.Updates():
+				if entry == nil { // Indicates all initial values are received
+					return
+				}
+				kl.keys <- entry.Key()
+			case <-ctx.Done():
+				return
+			}
+		}
+	}()
+
+	return kl, nil
+}
+
 func (kl *keyLister) Keys() <-chan string {
 	return kl.keys
 }
@@ -1383,7 +1416,7 @@ func mapStreamToKVS(js *jetStream, pushJS nats.JetStreamContext, stream Stream)
 		pushJS:     pushJS,
 		stream:     stream,
 		// Determine if we need to use the JS prefix in front of Put and Delete operations
-		useJSPfx:  js.apiPrefix != DefaultAPIPrefix,
+		useJSPfx:  js.opts.apiPrefix != DefaultAPIPrefix,
 		useDirect: info.Config.AllowDirect,
 	}
 
