@@ -1,4 +1,4 @@
-// Copyright 2022-2024 The NATS Authors
+// Copyright 2022-2025 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -81,12 +81,12 @@ type (
 
 	pullConsumer struct {
 		sync.Mutex
-		jetStream *jetStream
-		stream    string
-		durable   bool
-		name      string
-		info      *ConsumerInfo
-		subs      syncx.Map[string, *pullSubscription]
+		js      *jetStream
+		stream  string
+		durable bool
+		name    string
+		info    *ConsumerInfo
+		subs    syncx.Map[string, *pullSubscription]
 	}
 
 	pullRequest struct {
@@ -101,6 +101,7 @@ type (
 		Expires                 time.Duration
 		MaxMessages             int
 		MaxBytes                int
+		LimitSize               bool
 		Heartbeat               time.Duration
 		ErrHandler              ConsumeErrHandlerFunc
 		ReportMissingHeartbeats bool
@@ -160,9 +161,10 @@ type (
 )
 
 const (
-	DefaultMaxMessages = 500
-	DefaultExpires     = 30 * time.Second
-	unset              = -1
+	DefaultMaxMessages       = 500
+	DefaultExpires           = 30 * time.Second
+	defaultBatchMaxBytesOnly = 1_000_000
+	unset                    = -1
 )
 
 func min(x, y int) int {
@@ -187,18 +189,18 @@ func (p *pullConsumer) Consume(handler MessageHandler, opts ...PullConsumeOpt) (
 	}
 	p.Lock()
 
-	subject := apiSubj(p.jetStream.apiPrefix, fmt.Sprintf(apiRequestNextT, p.stream, p.name))
+	subject := p.js.apiSubject(fmt.Sprintf(apiRequestNextT, p.stream, p.name))
 
 	consumeID := nuid.Next()
 	sub := &pullSubscription{
 		id:          consumeID,
 		consumer:    p,
-		errs:        make(chan error, 1),
+		errs:        make(chan error, 10),
 		done:        make(chan struct{}, 1),
 		fetchNext:   make(chan *pullRequest, 1),
 		consumeOpts: consumeOpts,
 	}
-	sub.connStatusChanged = p.jetStream.conn.StatusChanged(nats.CONNECTED, nats.RECONNECTING)
+	sub.connStatusChanged = p.js.conn.StatusChanged(nats.CONNECTED, nats.RECONNECTING)
 
 	sub.hbMonitor = sub.scheduleHeartbeatCheck(consumeOpts.Heartbeat)
 
@@ -245,7 +247,7 @@ func (p *pullConsumer) Consume(handler MessageHandler, opts ...PullConsumeOpt) (
 			}
 			return
 		}
-		handler(p.jetStream.toJSMsg(msg))
+		handler(p.js.toJSMsg(msg))
 		sub.Lock()
 		sub.decrementPendingMsgs(msg)
 		sub.incrementDeliveredMsgs()
@@ -255,8 +257,8 @@ func (p *pullConsumer) Consume(handler MessageHandler, opts ...PullConsumeOpt) (
 			sub.Stop()
 		}
 	}
-	inbox := p.jetStream.conn.NewInbox()
-	sub.subscription, err = p.jetStream.conn.Subscribe(inbox, internalHandler)
+	inbox := p.js.conn.NewInbox()
+	sub.subscription, err = p.js.conn.Subscribe(inbox, internalHandler)
 	if err != nil {
 		return nil, err
 	}
@@ -373,7 +375,7 @@ func (s *pullSubscription) resetPendingMsgs() {
 // lock should be held before calling this method
 func (s *pullSubscription) decrementPendingMsgs(msg *nats.Msg) {
 	s.pending.msgCount--
-	if s.consumeOpts.MaxBytes != 0 {
+	if s.consumeOpts.MaxBytes != 0 && !s.consumeOpts.LimitSize {
 		s.pending.byteCount -= msg.Size()
 	}
 }
@@ -388,18 +390,23 @@ func (s *pullSubscription) incrementDeliveredMsgs() {
 // the buffer to trigger a new pull request.
 // lock should be held before calling this method
 func (s *pullSubscription) checkPending() {
+	// check if we went below any threshold
+	// we don't want to track bytes threshold if either it's not set or we used
+	// PullMaxMessagesWithBytesLimit
 	if (s.pending.msgCount < s.consumeOpts.ThresholdMessages ||
-		(s.pending.byteCount < s.consumeOpts.ThresholdBytes && s.consumeOpts.MaxBytes != 0)) &&
+		(s.pending.byteCount < s.consumeOpts.ThresholdBytes && s.consumeOpts.MaxBytes != 0 && !s.consumeOpts.LimitSize)) &&
 		s.fetchInProgress.Load() == 0 {
 
 		var batchSize, maxBytes int
-		if s.consumeOpts.MaxBytes == 0 {
-			// if using messages, calculate appropriate batch size
-			batchSize = s.consumeOpts.MaxMessages - s.pending.msgCount
-		} else {
-			// if using bytes, use the max value
-			batchSize = s.consumeOpts.MaxMessages
-			maxBytes = s.consumeOpts.MaxBytes - s.pending.byteCount
+		batchSize = s.consumeOpts.MaxMessages - s.pending.msgCount
+		if s.consumeOpts.MaxBytes != 0 {
+			if s.consumeOpts.LimitSize {
+				maxBytes = s.consumeOpts.MaxBytes
+			} else {
+				maxBytes = s.consumeOpts.MaxBytes - s.pending.byteCount
+				// when working with max bytes only, always ask for full batch
+				batchSize = s.consumeOpts.MaxMessages
+			}
 		}
 		if s.consumeOpts.StopAfter > 0 {
 			batchSize = min(batchSize, s.consumeOpts.StopAfter-s.delivered-s.pending.msgCount)
@@ -430,7 +437,7 @@ func (p *pullConsumer) Messages(opts ...PullMessagesOpt) (MessagesContext, error
 	}
 
 	p.Lock()
-	subject := apiSubj(p.jetStream.apiPrefix, fmt.Sprintf(apiRequestNextT, p.stream, p.name))
+	subject := p.js.apiSubject(fmt.Sprintf(apiRequestNextT, p.stream, p.name))
 
 	msgs := make(chan *nats.Msg, consumeOpts.MaxMessages)
 
@@ -440,13 +447,13 @@ func (p *pullConsumer) Messages(opts ...PullMessagesOpt) (MessagesContext, error
 		consumer:    p,
 		done:        make(chan struct{}, 1),
 		msgs:        msgs,
-		errs:        make(chan error, 1),
+		errs:        make(chan error, 10),
 		fetchNext:   make(chan *pullRequest, 1),
 		consumeOpts: consumeOpts,
 	}
-	sub.connStatusChanged = p.jetStream.conn.StatusChanged(nats.CONNECTED, nats.RECONNECTING)
-	inbox := p.jetStream.conn.NewInbox()
-	sub.subscription, err = p.jetStream.conn.ChanSubscribe(inbox, sub.msgs)
+	sub.connStatusChanged = p.js.conn.StatusChanged(nats.CONNECTED, nats.RECONNECTING)
+	inbox := p.js.conn.NewInbox()
+	sub.subscription, err = p.js.conn.ChanSubscribe(inbox, sub.msgs)
 	if err != nil {
 		p.Unlock()
 		return nil, err
@@ -507,7 +514,7 @@ func (s *pullSubscription) Next() (Msg, error) {
 	if closed && !drainMode {
 		return nil, ErrMsgIteratorClosed
 	}
-	hbMonitor := s.scheduleHeartbeatCheck(2 * s.consumeOpts.Heartbeat)
+	hbMonitor := s.scheduleHeartbeatCheck(s.consumeOpts.Heartbeat)
 	defer func() {
 		if hbMonitor != nil {
 			hbMonitor.Stop()
@@ -547,7 +554,7 @@ func (s *pullSubscription) Next() (Msg, error) {
 			}
 			s.decrementPendingMsgs(msg)
 			s.incrementDeliveredMsgs()
-			return s.consumer.jetStream.toJSMsg(msg), nil
+			return s.consumer.js.toJSMsg(msg), nil
 		case err := <-s.errs:
 			if errors.Is(err, ErrNoHeartbeat) {
 				s.pending.msgCount = 0
@@ -584,7 +591,7 @@ func (s *pullSubscription) Next() (Msg, error) {
 }
 
 func (s *pullSubscription) handleStatusMsg(msg *nats.Msg, msgErr error) error {
-	if !errors.Is(msgErr, nats.ErrTimeout) && !errors.Is(msgErr, ErrMaxBytesExceeded) {
+	if !errors.Is(msgErr, nats.ErrTimeout) && !errors.Is(msgErr, ErrMaxBytesExceeded) && !errors.Is(msgErr, ErrBatchCompleted) {
 		if errors.Is(msgErr, ErrConsumerDeleted) || errors.Is(msgErr, ErrBadRequest) {
 			return msgErr
 		}
@@ -605,7 +612,7 @@ func (s *pullSubscription) handleStatusMsg(msg *nats.Msg, msgErr error) error {
 	if s.pending.msgCount < 0 {
 		s.pending.msgCount = 0
 	}
-	if s.consumeOpts.MaxBytes > 0 {
+	if s.consumeOpts.MaxBytes > 0 && !s.consumeOpts.LimitSize {
 		s.pending.byteCount -= bytesLeft
 		if s.pending.byteCount < 0 {
 			s.pending.byteCount = 0
@@ -712,7 +719,7 @@ func (p *pullConsumer) Fetch(batch int, opts ...FetchOpt) (MessageBatch, error)
 // FetchBytes is used to retrieve up to a provided bytes from the stream.
 func (p *pullConsumer) FetchBytes(maxBytes int, opts ...FetchOpt) (MessageBatch, error) {
 	req := &pullRequest{
-		Batch:     1000000,
+		Batch:     defaultBatchMaxBytesOnly,
 		MaxBytes:  maxBytes,
 		Expires:   DefaultExpires,
 		Heartbeat: unset,
@@ -755,17 +762,17 @@ func (p *pullConsumer) fetch(req *pullRequest) (MessageBatch, error) {
 		msgs: make(chan Msg, req.Batch),
 	}
 	msgs := make(chan *nats.Msg, 2*req.Batch)
-	subject := apiSubj(p.jetStream.apiPrefix, fmt.Sprintf(apiRequestNextT, p.stream, p.name))
+	subject := p.js.apiSubject(fmt.Sprintf(apiRequestNextT, p.stream, p.name))
 
 	sub := &pullSubscription{
 		consumer: p,
 		done:     make(chan struct{}, 1),
 		msgs:     msgs,
-		errs:     make(chan error, 1),
+		errs:     make(chan error, 10),
 	}
-	inbox := p.jetStream.conn.NewInbox()
+	inbox := p.js.conn.NewInbox()
 	var err error
-	sub.subscription, err = p.jetStream.conn.ChanSubscribe(inbox, sub.msgs)
+	sub.subscription, err = p.js.conn.ChanSubscribe(inbox, sub.msgs)
 	if err != nil {
 		return nil, err
 	}
@@ -799,7 +806,7 @@ func (p *pullConsumer) fetch(req *pullRequest) (MessageBatch, error) {
 					res.Unlock()
 					continue
 				}
-				res.msgs <- p.jetStream.toJSMsg(msg)
+				res.msgs <- p.js.toJSMsg(msg)
 				meta, err := msg.Metadata()
 				if err != nil {
 					res.err = fmt.Errorf("parsing message metadata: %s", err)
@@ -938,7 +945,7 @@ func (s *pullSubscription) pull(req *pullRequest, subject string) error {
 	}
 
 	reply := s.subscription.Subject
-	if err := s.consumer.jetStream.conn.PublishRequest(subject, reply, reqJSON); err != nil {
+	if err := s.consumer.js.conn.PublishRequest(subject, reply, reqJSON); err != nil {
 		return err
 	}
 	return nil
@@ -985,40 +992,45 @@ func parseMessagesOpts(ordered bool, opts ...PullMessagesOpt) (*consumeOpts, err
 }
 
 func (consumeOpts *consumeOpts) setDefaults(ordered bool) error {
-	if consumeOpts.MaxBytes != unset && consumeOpts.MaxMessages != unset {
+	// we cannot use both max messages and max bytes unless we're using max bytes as fetch size limiter
+	if consumeOpts.MaxBytes != unset && consumeOpts.MaxMessages != unset && !consumeOpts.LimitSize {
 		return errors.New("only one of MaxMessages and MaxBytes can be specified")
 	}
-	if consumeOpts.MaxBytes != unset {
-		// when max_bytes is used, set batch size to a very large number
-		consumeOpts.MaxMessages = 1000000
-	} else if consumeOpts.MaxMessages != unset {
+	if consumeOpts.MaxBytes != unset && !consumeOpts.LimitSize {
+		// we used PullMaxBytes setting, set MaxMessages to a high value
+		consumeOpts.MaxMessages = defaultBatchMaxBytesOnly
+	} else if consumeOpts.MaxMessages == unset {
+		// otherwise, if max messages is not set, set it to default value
+		consumeOpts.MaxMessages = DefaultMaxMessages
+	}
+	// if user did not set max bytes, set it to 0
+	if consumeOpts.MaxBytes == unset {
 		consumeOpts.MaxBytes = 0
-	} else {
-		if consumeOpts.MaxBytes == unset {
-			consumeOpts.MaxBytes = 0
-		}
-		if consumeOpts.MaxMessages == unset {
-			consumeOpts.MaxMessages = DefaultMaxMessages
-		}
 	}
 
 	if consumeOpts.ThresholdMessages == 0 {
+		// half of the max messages, rounded up
 		consumeOpts.ThresholdMessages = int(math.Ceil(float64(consumeOpts.MaxMessages) / 2))
 	}
 	if consumeOpts.ThresholdBytes == 0 {
+		// half of the max bytes, rounded up
 		consumeOpts.ThresholdBytes = int(math.Ceil(float64(consumeOpts.MaxBytes) / 2))
 	}
+
+	// set default heartbeats
 	if consumeOpts.Heartbeat == unset {
+		// by default, use 50% of expiry time
+		consumeOpts.Heartbeat = consumeOpts.Expires / 2
 		if ordered {
-			consumeOpts.Heartbeat = 5 * time.Second
+			// for ordered consumers, the default heartbeat is 5 seconds
 			if consumeOpts.Expires < 10*time.Second {
 				consumeOpts.Heartbeat = consumeOpts.Expires / 2
+			} else {
+				consumeOpts.Heartbeat = 5 * time.Second
 			}
-		} else {
-			consumeOpts.Heartbeat = consumeOpts.Expires / 2
-			if consumeOpts.Heartbeat > 30*time.Second {
-				consumeOpts.Heartbeat = 30 * time.Second
-			}
+		} else if consumeOpts.Heartbeat > 30*time.Second {
+			// cap the heartbeat to 30 seconds
+			consumeOpts.Heartbeat = 30 * time.Second
 		}
 	}
 	if consumeOpts.Heartbeat > consumeOpts.Expires/2 {
