@@ -119,9 +119,9 @@ type (
 	}
 
 	stream struct {
-		name      string
-		info      *StreamInfo
-		jetStream *jetStream
+		name string
+		info *StreamInfo
+		js   *jetStream
 	}
 
 	// StreamInfoOpt is a function setting options for [Stream.Info]
@@ -241,7 +241,7 @@ type (
 // possible). Consumer interface is returned, allowing to operate on a
 // consumer (e.g. fetch messages).
 func (s *stream) CreateOrUpdateConsumer(ctx context.Context, cfg ConsumerConfig) (Consumer, error) {
-	return upsertConsumer(ctx, s.jetStream, s.name, cfg, consumerActionCreateOrUpdate)
+	return upsertConsumer(ctx, s.js, s.name, cfg, consumerActionCreateOrUpdate)
 }
 
 // CreateConsumer creates a consumer on a given stream with given
@@ -251,14 +251,14 @@ func (s *stream) CreateOrUpdateConsumer(ctx context.Context, cfg ConsumerConfig)
 // existing consumer is returned. Consumer interface is returned,
 // allowing to operate on a consumer (e.g. fetch messages).
 func (s *stream) CreateConsumer(ctx context.Context, cfg ConsumerConfig) (Consumer, error) {
-	return upsertConsumer(ctx, s.jetStream, s.name, cfg, consumerActionCreate)
+	return upsertConsumer(ctx, s.js, s.name, cfg, consumerActionCreate)
 }
 
 // UpdateConsumer updates an existing consumer. If consumer does not
 // exist, ErrConsumerDoesNotExist is returned. Consumer interface is
 // returned, allowing to operate on a consumer (e.g. fetch messages).
 func (s *stream) UpdateConsumer(ctx context.Context, cfg ConsumerConfig) (Consumer, error) {
-	return upsertConsumer(ctx, s.jetStream, s.name, cfg, consumerActionUpdate)
+	return upsertConsumer(ctx, s.js, s.name, cfg, consumerActionUpdate)
 }
 
 // OrderedConsumer returns an OrderedConsumer instance. OrderedConsumer
@@ -267,7 +267,7 @@ func (s *stream) UpdateConsumer(ctx context.Context, cfg ConsumerConfig) (Consum
 // pull consumers and are resilient to deletes and restarts.
 func (s *stream) OrderedConsumer(ctx context.Context, cfg OrderedConsumerConfig) (Consumer, error) {
 	oc := &orderedConsumer{
-		jetStream:  s.jetStream,
+		js:         s.js,
 		cfg:        &cfg,
 		stream:     s.name,
 		namePrefix: nuid.Next(),
@@ -287,18 +287,18 @@ func (s *stream) OrderedConsumer(ctx context.Context, cfg OrderedConsumerConfig)
 // of messages. If consumer does not exist, ErrConsumerNotFound is
 // returned.
 func (s *stream) Consumer(ctx context.Context, name string) (Consumer, error) {
-	return getConsumer(ctx, s.jetStream, s.name, name)
+	return getConsumer(ctx, s.js, s.name, name)
 }
 
 // DeleteConsumer removes a consumer with given name from a stream.
 // If consumer does not exist, ErrConsumerNotFound is returned.
 func (s *stream) DeleteConsumer(ctx context.Context, name string) error {
-	return deleteConsumer(ctx, s.jetStream, s.name, name)
+	return deleteConsumer(ctx, s.js, s.name, name)
 }
 
 // Info returns StreamInfo from the server.
 func (s *stream) Info(ctx context.Context, opts ...StreamInfoOpt) (*StreamInfo, error) {
-	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	ctx, cancel := s.js.wrapContextWithoutDeadline(ctx)
 	if cancel != nil {
 		defer cancel()
 	}
@@ -316,7 +316,7 @@ func (s *stream) Info(ctx context.Context, opts ...StreamInfoOpt) (*StreamInfo,
 	var subjectMap map[string]uint64
 	var offset int
 
-	infoSubject := apiSubj(s.jetStream.apiPrefix, fmt.Sprintf(apiStreamInfoT, s.name))
+	infoSubject := fmt.Sprintf(apiStreamInfoT, s.name)
 	var info *StreamInfo
 	for {
 		if infoReq != nil {
@@ -332,7 +332,7 @@ func (s *stream) Info(ctx context.Context, opts ...StreamInfoOpt) (*StreamInfo,
 			}
 		}
 		var resp streamInfoResponse
-		if _, err = s.jetStream.apiRequestJSON(ctx, infoSubject, &resp, req); err != nil {
+		if _, err = s.js.apiRequestJSON(ctx, infoSubject, &resp, req); err != nil {
 			return nil, err
 		}
 		if resp.Error != nil {
@@ -375,7 +375,7 @@ func (s *stream) CachedInfo() *StreamInfo {
 // Purge removes messages from a stream. It is a destructive operation.
 // Use with caution. See StreamPurgeOpt for available options.
 func (s *stream) Purge(ctx context.Context, opts ...StreamPurgeOpt) error {
-	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	ctx, cancel := s.js.wrapContextWithoutDeadline(ctx)
 	if cancel != nil {
 		defer cancel()
 	}
@@ -392,10 +392,10 @@ func (s *stream) Purge(ctx context.Context, opts ...StreamPurgeOpt) error {
 		return err
 	}
 
-	purgeSubject := apiSubj(s.jetStream.apiPrefix, fmt.Sprintf(apiStreamPurgeT, s.name))
+	purgeSubject := fmt.Sprintf(apiStreamPurgeT, s.name)
 
 	var resp streamPurgeResponse
-	if _, err = s.jetStream.apiRequestJSON(ctx, purgeSubject, &resp, req); err != nil {
+	if _, err = s.js.apiRequestJSON(ctx, purgeSubject, &resp, req); err != nil {
 		return err
 	}
 	if resp.Error != nil {
@@ -423,7 +423,7 @@ func (s *stream) GetLastMsgForSubject(ctx context.Context, subject string) (*Raw
 }
 
 func (s *stream) getMsg(ctx context.Context, mreq *apiMsgGetRequest) (*RawStreamMsg, error) {
-	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	ctx, cancel := s.js.wrapContextWithoutDeadline(ctx)
 	if cancel != nil {
 		defer cancel()
 	}
@@ -436,24 +436,24 @@ func (s *stream) getMsg(ctx context.Context, mreq *apiMsgGetRequest) (*RawStream
 	// handle direct gets
 	if s.info.Config.AllowDirect {
 		if mreq.LastFor != "" {
-			gmSubj = apiSubj(s.jetStream.apiPrefix, fmt.Sprintf(apiDirectMsgGetLastBySubjectT, s.name, mreq.LastFor))
-			r, err := s.jetStream.apiRequest(ctx, gmSubj, nil)
+			gmSubj = fmt.Sprintf(apiDirectMsgGetLastBySubjectT, s.name, mreq.LastFor)
+			r, err := s.js.apiRequest(ctx, gmSubj, nil)
 			if err != nil {
 				return nil, err
 			}
-			return convertDirectGetMsgResponseToMsg(s.name, r.msg)
+			return convertDirectGetMsgResponseToMsg(r.msg)
 		}
-		gmSubj = apiSubj(s.jetStream.apiPrefix, fmt.Sprintf(apiDirectMsgGetT, s.name))
-		r, err := s.jetStream.apiRequest(ctx, gmSubj, req)
+		gmSubj = fmt.Sprintf(apiDirectMsgGetT, s.name)
+		r, err := s.js.apiRequest(ctx, gmSubj, req)
 		if err != nil {
 			return nil, err
 		}
-		return convertDirectGetMsgResponseToMsg(s.name, r.msg)
+		return convertDirectGetMsgResponseToMsg(r.msg)
 	}
 
 	var resp apiMsgGetResponse
-	dsSubj := apiSubj(s.jetStream.apiPrefix, fmt.Sprintf(apiMsgGetT, s.name))
-	_, err = s.jetStream.apiRequestJSON(ctx, dsSubj, &resp, req)
+	dsSubj := fmt.Sprintf(apiMsgGetT, s.name)
+	_, err = s.js.apiRequestJSON(ctx, dsSubj, &resp, req)
 	if err != nil {
 		return nil, err
 	}
@@ -484,7 +484,7 @@ func (s *stream) getMsg(ctx context.Context, mreq *apiMsgGetRequest) (*RawStream
 	}, nil
 }
 
-func convertDirectGetMsgResponseToMsg(name string, r *nats.Msg) (*RawStreamMsg, error) {
+func convertDirectGetMsgResponseToMsg(r *nats.Msg) (*RawStreamMsg, error) {
 	// Check for 404/408. We would get a no-payload message and a "Status" header
 	if len(r.Data) == 0 {
 		val := r.Header.Get(statusHdr)
@@ -555,7 +555,7 @@ func (s *stream) SecureDeleteMsg(ctx context.Context, seq uint64) error {
 }
 
 func (s *stream) deleteMsg(ctx context.Context, req *msgDeleteRequest) error {
-	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	ctx, cancel := s.js.wrapContextWithoutDeadline(ctx)
 	if cancel != nil {
 		defer cancel()
 	}
@@ -563,9 +563,9 @@ func (s *stream) deleteMsg(ctx context.Context, req *msgDeleteRequest) error {
 	if err != nil {
 		return err
 	}
-	subj := apiSubj(s.jetStream.apiPrefix, fmt.Sprintf(apiMsgDeleteT, s.name))
+	subj := fmt.Sprintf(apiMsgDeleteT, s.name)
 	var resp msgDeleteResponse
-	if _, err = s.jetStream.apiRequestJSON(ctx, subj, &resp, r); err != nil {
+	if _, err = s.js.apiRequestJSON(ctx, subj, &resp, r); err != nil {
 		return err
 	}
 	if !resp.Success {
@@ -578,12 +578,12 @@ func (s *stream) deleteMsg(ctx context.Context, req *msgDeleteRequest) error {
 // channel of consumer infos.
 func (s *stream) ListConsumers(ctx context.Context) ConsumerInfoLister {
 	l := &consumerLister{
-		js:        s.jetStream,
+		js:        s.js,
 		consumers: make(chan *ConsumerInfo),
 	}
 	go func() {
 		defer close(l.consumers)
-		ctx, cancel := wrapContextWithoutDeadline(ctx)
+		ctx, cancel := s.js.wrapContextWithoutDeadline(ctx)
 		if cancel != nil {
 			defer cancel()
 		}
@@ -625,12 +625,12 @@ func (s *consumerLister) Err() error {
 // channel of consumer names.
 func (s *stream) ConsumerNames(ctx context.Context) ConsumerNameLister {
 	l := &consumerLister{
-		js:    s.jetStream,
+		js:    s.js,
 		names: make(chan string),
 	}
 	go func() {
 		defer close(l.names)
-		ctx, cancel := wrapContextWithoutDeadline(ctx)
+		ctx, cancel := s.js.wrapContextWithoutDeadline(ctx)
 		if cancel != nil {
 			defer cancel()
 		}
@@ -674,7 +674,7 @@ func (s *consumerLister) consumerInfos(ctx context.Context, stream string) ([]*C
 		return nil, err
 	}
 
-	slSubj := apiSubj(s.js.apiPrefix, fmt.Sprintf(apiConsumerListT, stream))
+	slSubj := fmt.Sprintf(apiConsumerListT, stream)
 	var resp consumerListResponse
 	_, err = s.js.apiRequestJSON(ctx, slSubj, &resp, req)
 	if err != nil {
@@ -702,7 +702,7 @@ func (s *consumerLister) consumerNames(ctx context.Context, stream string) ([]st
 		return nil, err
 	}
 
-	slSubj := apiSubj(s.js.apiPrefix, fmt.Sprintf(apiConsumerNamesT, stream))
+	slSubj := fmt.Sprintf(apiConsumerNamesT, stream)
 	var resp consumerNamesResponse
 	_, err = s.js.apiRequestJSON(ctx, slSubj, &resp, req)
 	if err != nil {
