@@ -1,4 +1,4 @@
-// Copyright 2023-2024 The NATS Authors
+// Copyright 2023-2025 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -1037,6 +1037,85 @@ func TestKeyValueListKeys(t *testing.T) {
 	}
 }
 
+func TestListKeysFiltered(t *testing.T) {
+	s := RunBasicJetStreamServer()
+	defer shutdownJSServerAndRemoveStorage(t, s)
+
+	nc, js := jsClient(t, s)
+	defer nc.Close()
+	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+	defer cancel()
+
+	// Create Key-Value store.
+	kv, err := js.CreateKeyValue(ctx, jetstream.KeyValueConfig{Bucket: "KVS", History: 2})
+	expectOk(t, err)
+
+	// Helper function to add key-value pairs.
+	putKeys := func(data map[string]string) {
+		for key, value := range data {
+			t.Helper()
+			_, err := kv.Put(ctx, key, []byte(value))
+			expectOk(t, err)
+		}
+	}
+
+	// Add key-value pairs.
+	putKeys(map[string]string{
+		"apple":  "fruit",
+		"banana": "fruit",
+		"carrot": "vegetable",
+	})
+
+	// Use filters to list keys matching "apple".
+	filters := []string{"apple"}
+	keyLister, err := kv.ListKeysFiltered(ctx, filters...)
+	expectOk(t, err)
+
+	// Collect filtered keys from KeyLister
+	var filteredKeys []string
+	for key := range keyLister.Keys() {
+		filteredKeys = append(filteredKeys, key)
+	}
+
+	// Validate expected keys.
+	expectedKeys := []string{"apple"}
+	if len(filteredKeys) != len(expectedKeys) {
+		t.Fatalf("Expected %d filtered key(s), got %d", len(expectedKeys), len(filteredKeys))
+	}
+
+	for _, key := range expectedKeys {
+		if !contains(filteredKeys, key) {
+			t.Fatalf("Expected key %s in filtered keys, but not found", key)
+		}
+	}
+
+	// delete apple so we can ensure we do not see it later after another filtered search
+	err = kv.Delete(ctx, "apple")
+	expectOk(t, err)
+
+	filters = []string{"apple"}
+	keyLister, err = kv.ListKeysFiltered(ctx, filters...)
+	expectOk(t, err)
+
+	// reset filtered keys
+	filteredKeys = nil
+	for key := range keyLister.Keys() {
+		filteredKeys = append(filteredKeys, key)
+	}
+	if len(filteredKeys) != 0 {
+		t.Fatalf("Expected 0 deleted keys, but %d was found", len(filteredKeys))
+	}
+}
+
+func contains(slice []string, key string) bool {
+	for _, k := range slice {
+		if k == key {
+			return true
+		}
+	}
+	return false
+}
+
 func TestKeyValueCrossAccounts(t *testing.T) {
 	conf := createConfFile(t, []byte(`
 		listen: 127.0.0.1:-1
@@ -1618,6 +1697,8 @@ func TestKeyValueCreate(t *testing.T) {
 	if err != nil {
 		t.Fatalf("Error getting stream: %v", err)
 	}
+	// server will set metadata values, so we need to clear them
+	stream.CachedInfo().Config.Metadata = nil
 	if !reflect.DeepEqual(stream.CachedInfo().Config, expectedStreamConfig) {
 		t.Fatalf("Expected stream config to be %+v, got %+v", expectedStreamConfig, stream.CachedInfo().Config)
 	}
