@@ -26,10 +26,6 @@ import (
 	"github.com/nats-io/nuid"
 )
 
-// Notice: Experimental Preview
-//
-// This functionality is EXPERIMENTAL and may be changed in later releases.
-
 type (
 
 	// Service exposes methods to operate on a service instance.
@@ -71,7 +67,7 @@ type (
 	}
 
 	EndpointOpt func(*endpointOpts) error
-	GroupOpt    func(*groupOpts) error
+	GroupOpt    func(*groupOpts)
 
 	endpointOpts struct {
 		subject    string
@@ -245,6 +241,7 @@ type (
 
 	asyncCallbacksHandler struct {
 		cbQueue chan func()
+		closed  bool
 	}
 )
 
@@ -435,6 +432,7 @@ func addEndpoint(s *service, name, subject string, handler Handler, metadata map
 	if err != nil {
 		return err
 	}
+	s.m.Lock()
 	endpoint.subscription = sub
 	s.endpoints = append(s.endpoints, endpoint)
 	endpoint.stats = EndpointStats{
@@ -442,6 +440,7 @@ func addEndpoint(s *service, name, subject string, handler Handler, metadata map
 		Subject:    subject,
 		QueueGroup: queueGroup,
 	}
+	s.m.Unlock()
 	return nil
 }
 
@@ -461,8 +460,8 @@ func (s *service) AddGroup(name string, opts ...GroupOpt) Group {
 // dispatch is responsible for calling any async callbacks
 func (ac *asyncCallbacksHandler) run() {
 	for {
-		f := <-ac.cbQueue
-		if f == nil {
+		f, ok := <-ac.cbQueue
+		if !ok || f == nil {
 			return
 		}
 		f()
@@ -475,7 +474,11 @@ func (ac *asyncCallbacksHandler) push(f func()) {
 }
 
 func (ac *asyncCallbacksHandler) close() {
+	if ac.closed {
+		return
+	}
 	close(ac.cbQueue)
+	ac.closed = true
 }
 
 func (c *Config) valid() error {
@@ -531,8 +534,11 @@ func (s *service) wrapConnectionEventCallbacks() {
 				endpoint.stats.LastError = err.Error()
 			}
 			s.m.Unlock()
-			s.Stop()
-			s.natsHandlers.asyncErr(c, sub, err)
+			if stopErr := s.Stop(); stopErr != nil {
+				s.natsHandlers.asyncErr(c, sub, errors.Join(err, fmt.Errorf("stopping service: %w", stopErr)))
+			} else {
+				s.natsHandlers.asyncErr(c, sub, err)
+			}
 		})
 	} else {
 		s.nc.SetErrorHandler(func(c *nats.Conn, sub *nats.Subscription, err error) {
@@ -561,6 +567,9 @@ func (s *service) wrapConnectionEventCallbacks() {
 }
 
 func unwrapConnectionEventCallbacks(nc *nats.Conn, handlers handlers) {
+	if nc.IsClosed() {
+		return
+	}
 	nc.SetClosedHandler(handlers.closed)
 	nc.SetErrorHandler(handlers.asyncErr)
 }
@@ -618,7 +627,9 @@ func (svc *service) addVerbHandlers(nc *nats.Conn, verb Verb, handler HandlerFun
 func (s *service) addInternalHandler(nc *nats.Conn, verb Verb, kind, id, name string, handler HandlerFunc) error {
 	subj, err := ControlSubject(verb, kind, id)
 	if err != nil {
-		s.Stop()
+		if stopErr := s.Stop(); stopErr != nil {
+			return errors.Join(err, fmt.Errorf("stopping service: %w", stopErr))
+		}
 		return err
 	}
 
@@ -626,7 +637,9 @@ func (s *service) addInternalHandler(nc *nats.Conn, verb Verb, kind, id, name st
 		handler(&request{msg: msg})
 	})
 	if err != nil {
-		s.Stop()
+		if stopErr := s.Stop(); stopErr != nil {
+			return errors.Join(err, fmt.Errorf("stopping service: %w", stopErr))
+		}
 		return err
 	}
 	return nil
@@ -658,6 +671,7 @@ func (s *service) Stop() error {
 	}
 	for _, e := range s.endpoints {
 		if err := e.stop(); err != nil {
+			fmt.Println("Error stopping endpoint: ", err)
 			return err
 		}
 	}
@@ -665,6 +679,10 @@ func (s *service) Stop() error {
 	for key, sub := range s.verbSubs {
 		keys = append(keys, key)
 		if err := sub.Drain(); err != nil {
+			// connection is closed so draining is not possible
+			if errors.Is(err, nats.ErrConnectionClosed) {
+				break
+			}
 			return fmt.Errorf("draining subscription for subject %q: %w", sub.Subject, err)
 		}
 	}
@@ -675,8 +693,8 @@ func (s *service) Stop() error {
 	s.stopped = true
 	if s.DoneHandler != nil {
 		s.asyncDispatcher.push(func() { s.DoneHandler(s) })
-		s.asyncDispatcher.close()
 	}
+	s.asyncDispatcher.close()
 	return nil
 }
 
@@ -691,6 +709,9 @@ func (s *service) serviceIdentity() ServiceIdentity {
 
 // Info returns information about the service
 func (s *service) Info() Info {
+	s.m.Lock()
+	defer s.m.Unlock()
+
 	endpoints := make([]EndpointInfo, 0, len(s.endpoints))
 	for _, e := range s.endpoints {
 		endpoints = append(endpoints, EndpointInfo{
@@ -817,7 +838,9 @@ func (g *group) AddGroup(name string, opts ...GroupOpt) Group {
 }
 
 func (e *Endpoint) stop() error {
-	if err := e.subscription.Drain(); err != nil {
+	// Drain the subscription. If the connection is closed, draining is not possible
+	// but we should still remove the endpoint from the service.
+	if err := e.subscription.Drain(); err != nil && !errors.Is(err, nats.ErrConnectionClosed) {
 		return fmt.Errorf("draining subscription for request handler: %w", err)
 	}
 	for i := 0; i < len(e.service.endpoints); i++ {
@@ -885,8 +908,7 @@ func WithEndpointQueueGroup(queueGroup string) EndpointOpt {
 }
 
 func WithGroupQueueGroup(queueGroup string) GroupOpt {
-	return func(g *groupOpts) error {
+	return func(g *groupOpts) {
 		g.queueGroup = queueGroup
-		return nil
 	}
 }
