@@ -1,4 +1,4 @@
-// Copyright 2022 The NATS Authors
+// Copyright 2022-2025 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -79,7 +79,7 @@ func TestObjectBasics(t *testing.T) {
 	status, err := obs.Status()
 	expectOk(t, err)
 	if !status.Sealed() {
-		t.Fatalf("exected sealed status")
+		t.Fatalf("expected sealed status")
 	}
 	if status.Size() == 0 {
 		t.Fatalf("size is 0")
@@ -401,27 +401,47 @@ func TestObjectMetadata(t *testing.T) {
 	nc, js := jsClient(t, s)
 	defer nc.Close()
 
-	obs, err := js.CreateObjectStore(&nats.ObjectStoreConfig{Bucket: "META-TEST"})
+	bucketMetadata := map[string]string{"foo": "bar", "baz": "boo"}
+	obs, err := js.CreateObjectStore(&nats.ObjectStoreConfig{
+		Bucket:   "META-TEST",
+		Metadata: bucketMetadata,
+	})
 	expectOk(t, err)
+	status, err := obs.Status()
+	expectOk(t, err)
+	for k, v := range bucketMetadata {
+		if status.Metadata()[k] != v {
+			t.Fatalf("invalid bucket metadata: %+v", status.Metadata())
+		}
+	}
 
 	// Simple with no Meta.
 	_, err = obs.PutString("A", "AAA")
 	expectOk(t, err)
-	_, err = obs.PutString("C", "CCC")
+	buf := bytes.NewBufferString("CCC")
+	objectMetadata := map[string]string{"name": "C", "description": "descC"}
+	info, err := obs.Put(&nats.ObjectMeta{Name: "C", Metadata: objectMetadata}, buf)
 	expectOk(t, err)
+	if !reflect.DeepEqual(info.Metadata, objectMetadata) {
+		t.Fatalf("invalid object metadata: %+v", info.Metadata)
+	}
 
 	meta := &nats.ObjectMeta{Name: "A"}
 	meta.Description = "descA"
 	meta.Headers = make(nats.Header)
 	meta.Headers.Set("color", "blue")
+	objectMetadata["description"] = "updated desc"
+	objectMetadata["version"] = "0.1"
+	meta.Metadata = objectMetadata
 
 	// simple update that does not change the name, just adds data
 	err = obs.UpdateMeta("A", meta)
 	expectOk(t, err)
 
-	info, err := obs.GetInfo("A")
+	info, err = obs.GetInfo("A")
 	expectOk(t, err)
-	if info.Name != "A" || info.Description != "descA" || info.Headers == nil || info.Headers.Get("color") != "blue" {
+	if info.Name != "A" || info.Description != "descA" || info.Headers == nil || info.Headers.Get("color") != "blue" ||
+		!reflect.DeepEqual(info.Metadata, objectMetadata) {
 		t.Fatalf("Update failed: %+v", info)
 	}
 
@@ -430,6 +450,7 @@ func TestObjectMetadata(t *testing.T) {
 	meta.Description = "descB"
 	meta.Headers = make(nats.Header)
 	meta.Headers.Set("color", "red")
+	meta.Metadata = nil
 
 	err = obs.UpdateMeta("A", meta)
 	expectOk(t, err)
@@ -441,7 +462,7 @@ func TestObjectMetadata(t *testing.T) {
 
 	info, err = obs.GetInfo("B")
 	expectOk(t, err)
-	if info.Name != "B" || info.Description != "descB" || info.Headers == nil || info.Headers.Get("color") != "red" {
+	if info.Name != "B" || info.Description != "descB" || info.Headers == nil || info.Headers.Get("color") != "red" || info.Metadata != nil {
 		t.Fatalf("Update failed: %+v", info)
 	}
 
@@ -474,86 +495,136 @@ func TestObjectMetadata(t *testing.T) {
 }
 
 func TestObjectWatch(t *testing.T) {
-	s := RunBasicJetStreamServer()
-	defer shutdownJSServerAndRemoveStorage(t, s)
-
-	nc, js := jsClient(t, s)
-	defer nc.Close()
-
-	obs, err := js.CreateObjectStore(&nats.ObjectStoreConfig{Bucket: "WATCH-TEST"})
-	expectOk(t, err)
-
-	watcher, err := obs.Watch()
-	expectOk(t, err)
-	defer watcher.Stop()
-
-	expectUpdate := func(name string) {
-		t.Helper()
-		select {
-		case info := <-watcher.Updates():
-			if false && info.Name != name { // TODO what is supposed to happen here?
-				t.Fatalf("Expected update for %q, but got %+v", name, info)
+	expectUpdateF := func(t *testing.T, watcher nats.ObjectWatcher) func(name string) {
+		return func(name string) {
+			t.Helper()
+			select {
+			case info := <-watcher.Updates():
+				if false && info.Name != name { // TODO what is supposed to happen here?
+					t.Fatalf("Expected update for %q, but got %+v", name, info)
+				}
+			case <-time.After(time.Second):
+				t.Fatalf("Did not receive an update like expected")
 			}
-		case <-time.After(time.Second):
-			t.Fatalf("Did not receive an update like expected")
 		}
 	}
 
-	expectNoMoreUpdates := func() {
-		t.Helper()
-		select {
-		case info := <-watcher.Updates():
-			t.Fatalf("Got an unexpected update: %+v", info)
-		case <-time.After(100 * time.Millisecond):
-		}
-	}
-
-	expectInitDone := func() {
-		t.Helper()
-		select {
-		case info := <-watcher.Updates():
-			if info != nil {
-				t.Fatalf("Did not get expected: %+v", info)
+	expectNoMoreUpdatesF := func(t *testing.T, watcher nats.ObjectWatcher) func() {
+		return func() {
+			t.Helper()
+			select {
+			case info := <-watcher.Updates():
+				t.Fatalf("Got an unexpected update: %+v", info)
+			case <-time.After(100 * time.Millisecond):
 			}
-		case <-time.After(time.Second):
-			t.Fatalf("Did not receive a init done like expected")
 		}
 	}
 
-	// We should get a marker that is nil when all initital values are delivered.
-	expectInitDone()
-
-	_, err = obs.PutString("A", "AAA")
-	expectOk(t, err)
-	_, err = obs.PutString("B", "BBB")
-	expectOk(t, err)
-
-	// Initial Values.
-	expectUpdate("A")
-	expectUpdate("B")
-	expectNoMoreUpdates()
-
-	// Delete
-	err = obs.Delete("A")
-	expectOk(t, err)
-
-	expectUpdate("A")
-	expectNoMoreUpdates()
-
-	// New
-	_, err = obs.PutString("C", "CCC")
-	expectOk(t, err)
-
-	// Update Meta
-	deletedInfo, err := obs.GetInfo("A", nats.GetObjectInfoShowDeleted())
-	expectOk(t, err)
-	if !deletedInfo.Deleted {
-		t.Fatalf("Expected object to be deleted.")
+	expectInitDoneF := func(t *testing.T, watcher nats.ObjectWatcher) func() {
+		return func() {
+			t.Helper()
+			select {
+			case info := <-watcher.Updates():
+				if info != nil {
+					t.Fatalf("Did not get expected: %+v", info)
+				}
+			case <-time.After(time.Second):
+				t.Fatalf("Did not receive a init done like expected")
+			}
+		}
 	}
-	meta := &deletedInfo.ObjectMeta
-	meta.Description = "Making a change."
-	err = obs.UpdateMeta("A", meta)
-	expectErr(t, err, nats.ErrUpdateMetaDeleted)
+
+	t.Run("default watcher", func(t *testing.T) {
+		s := RunBasicJetStreamServer()
+		defer shutdownJSServerAndRemoveStorage(t, s)
+
+		nc, js := jsClient(t, s)
+		defer nc.Close()
+
+		obs, err := js.CreateObjectStore(&nats.ObjectStoreConfig{Bucket: "WATCH-TEST"})
+		expectOk(t, err)
+
+		watcher, err := obs.Watch()
+		expectOk(t, err)
+		defer watcher.Stop()
+
+		expectUpdate := expectUpdateF(t, watcher)
+		expectNoMoreUpdates := expectNoMoreUpdatesF(t, watcher)
+		expectInitDone := expectInitDoneF(t, watcher)
+
+		// We should get a marker that is nil when all initial values are delivered.
+		expectInitDone()
+
+		_, err = obs.PutString("A", "AAA")
+		expectOk(t, err)
+		_, err = obs.PutString("B", "BBB")
+		expectOk(t, err)
+
+		// Initial Values.
+		expectUpdate("A")
+		expectUpdate("B")
+		expectNoMoreUpdates()
+
+		// Delete
+		err = obs.Delete("A")
+		expectOk(t, err)
+
+		expectUpdate("A")
+		expectNoMoreUpdates()
+
+		// New
+		_, err = obs.PutString("C", "CCC")
+		expectOk(t, err)
+
+		// Update Meta
+		deletedInfo, err := obs.GetInfo("A", nats.GetObjectInfoShowDeleted())
+		expectOk(t, err)
+		if !deletedInfo.Deleted {
+			t.Fatalf("Expected object to be deleted.")
+		}
+		meta := &deletedInfo.ObjectMeta
+		meta.Description = "Making a change."
+		err = obs.UpdateMeta("A", meta)
+		expectErr(t, err, nats.ErrUpdateMetaDeleted)
+	})
+
+	t.Run("watcher with update", func(t *testing.T) {
+		s := RunBasicJetStreamServer()
+		defer shutdownJSServerAndRemoveStorage(t, s)
+
+		nc, js := jsClient(t, s)
+		defer nc.Close()
+
+		obs, err := js.CreateObjectStore(&nats.ObjectStoreConfig{Bucket: "WATCH-TEST"})
+		expectOk(t, err)
+
+		_, err = obs.PutString("A", "AAA")
+		expectOk(t, err)
+		_, err = obs.PutString("B", "BBB")
+		expectOk(t, err)
+
+		watcher, err := obs.Watch(nats.UpdatesOnly())
+		expectOk(t, err)
+		defer watcher.Stop()
+
+		expectUpdate := expectUpdateF(t, watcher)
+		expectNoMoreUpdates := expectNoMoreUpdatesF(t, watcher)
+
+		// when listening for updates only, we should not receive anything when watcher is started
+		expectNoMoreUpdates()
+
+		// Delete
+		err = obs.Delete("A")
+		expectOk(t, err)
+
+		expectUpdate("A")
+		expectNoMoreUpdates()
+
+		// New
+		_, err = obs.PutString("C", "CCC")
+		expectOk(t, err)
+		expectUpdate("C")
+	})
 }
 
 func TestObjectLinks(t *testing.T) {
@@ -823,22 +894,15 @@ func TestObjectList(t *testing.T) {
 			t.Fatalf("Expected 4 total objects, got %d", len(omap))
 		}
 		expected := map[string]struct{}{
-			"A": struct{}{},
-			"B": struct{}{},
-			"C": struct{}{},
-			"b": struct{}{},
+			"A": {},
+			"B": {},
+			"C": {},
+			"b": {},
 		}
 		if !reflect.DeepEqual(omap, expected) {
 			t.Fatalf("Expected %+v but got %+v", expected, omap)
 		}
 	})
-
-	t.Run("context timeout", func(t *testing.T) {
-		ctx, cancel := context.WithTimeout(context.Background(), 1*time.Nanosecond)
-		defer cancel()
-		_, err := root.List(nats.Context(ctx))
-		expectErr(t, err, context.DeadlineExceeded)
-	})
 }
 
 func TestObjectMaxBytes(t *testing.T) {
@@ -989,3 +1053,134 @@ func TestDecodeObjectDigest(t *testing.T) {
 		})
 	}
 }
+
+func TestObjectStoreGetObjectContextTimeout(t *testing.T) {
+	s := RunBasicJetStreamServer()
+	defer shutdownJSServerAndRemoveStorage(t, s)
+
+	nc, js := jsClient(t, s)
+	defer nc.Close()
+
+	obs, err := js.CreateObjectStore(&nats.ObjectStoreConfig{Bucket: "OBJS"})
+	expectOk(t, err)
+
+	blob := make([]byte, 1024)
+	_, err = rand.Read(blob)
+	expectOk(t, err)
+
+	_, err = obs.PutBytes("blob", blob)
+	expectOk(t, err)
+
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Millisecond)
+	defer cancel()
+
+	r, err := obs.Get("blob", nats.Context(ctx))
+	expectOk(t, err)
+	time.Sleep(15 * time.Millisecond)
+	var res []byte
+	_, err = r.Read(res)
+	expectErr(t, err, nats.ErrTimeout)
+	r.Close()
+}
+
+func TestObjectStoreCompression(t *testing.T) {
+	s := RunBasicJetStreamServer()
+	defer shutdownJSServerAndRemoveStorage(t, s)
+
+	nc, js := jsClient(t, s)
+	defer nc.Close()
+
+	obj, err := js.CreateObjectStore(&nats.ObjectStoreConfig{
+		Bucket:      "A",
+		Compression: true,
+	})
+	if err != nil {
+		t.Fatalf("Error creating object store: %v", err)
+	}
+
+	status, err := obj.Status()
+	if err != nil {
+		t.Fatalf("Error getting bucket status: %v", err)
+	}
+
+	if !status.IsCompressed() {
+		t.Fatalf("Expected bucket to be compressed")
+	}
+
+	objStream, err := js.StreamInfo("OBJ_A")
+	if err != nil {
+		t.Fatalf("Error getting stream info: %v", err)
+	}
+
+	if objStream.Config.Compression != nats.S2Compression {
+		t.Fatalf("Expected stream to be compressed with S2")
+	}
+}
+
+func TestObjectStoreMirror(t *testing.T) {
+	s := RunBasicJetStreamServer()
+	defer shutdownJSServerAndRemoveStorage(t, s)
+
+	nc, js := jsClient(t, s)
+	defer nc.Close()
+
+	bucketName := "test-bucket"
+
+	obs, err := js.CreateObjectStore(&nats.ObjectStoreConfig{Bucket: bucketName, Description: "testing"})
+	expectOk(t, err)
+
+	mirrorBucketName := "mirror-test-bucket"
+
+	_, err = js.AddStream(&nats.StreamConfig{
+		Name: fmt.Sprintf("OBJ_%s", mirrorBucketName),
+		Mirror: &nats.StreamSource{
+			Name: fmt.Sprintf("OBJ_%s", bucketName),
+			SubjectTransforms: []nats.SubjectTransformConfig{
+				{
+					Source:      fmt.Sprintf("$O.%s.>", bucketName),
+					Destination: fmt.Sprintf("$O.%s.>", mirrorBucketName),
+				},
+			},
+		},
+		AllowRollup: true, // meta messages are always rollups
+	})
+	if err != nil {
+		t.Fatalf("Error creating object store bucket mirror: %v", err)
+	}
+
+	_, err = obs.PutString("A", "abc")
+	expectOk(t, err)
+
+	mirrorObs, err := js.ObjectStore(mirrorBucketName)
+	expectOk(t, err)
+
+	// Make sure we sync.
+	checkFor(t, 2*time.Second, 15*time.Millisecond, func() error {
+		mirrorValue, err := mirrorObs.GetString("A")
+		if err != nil {
+			return err
+		}
+		if mirrorValue != "abc" {
+			t.Fatalf("Expected mirrored object store value to be the same as original")
+		}
+		return nil
+	})
+
+	watcher, err := mirrorObs.Watch()
+	if err != nil {
+		t.Fatalf("Error creating watcher: %v", err)
+	}
+	defer watcher.Stop()
+
+	// expect to get one value and nil
+	for {
+		select {
+		case info := <-watcher.Updates():
+			if info == nil {
+				return
+			}
+		case <-time.After(2 * time.Second):
+			t.Fatalf("Expected to receive an update")
+		}
+	}
+}
